package com.sf.vertx.init;

import cn.hutool.core.util.StrUtil;
import io.vertx.core.AsyncResult;
import io.vertx.core.Future;
import io.vertx.core.Handler;
import io.vertx.core.Vertx;
import io.vertx.core.json.JsonArray;
import io.vertx.core.json.JsonObject;
import io.vertx.redis.client.*;
import io.vertx.redis.client.impl.RedisClient;

import java.util.Objects;
import java.util.UUID;

/**
 * 功能描述:
 *
 * @author a_kun
 * @date 2024/5/27 16:53
 */
public class RedisManager {

    private static RedisAPI redisAPI;
    private static Redis redis;

    private static RedisProperties redisProperties;

    public static void initRedisConfig(Vertx vertx, JsonObject redisConfig, Handler<AsyncResult<RedisConnection>> handler) {
        try {
            if (redis == null) {
                redisProperties = redisConfig.mapTo(RedisProperties.class);
                //解析配置
                RedisOptions options = new RedisOptions()
                        .setType(redisProperties.getClientType())
                        .setPoolName(redisProperties.getPoolName())
                        .setMaxPoolSize(redisProperties.getMaxPoolSize())
                        .setMaxPoolWaiting(redisProperties.getMaxPoolWaiting())
                        .setPoolCleanerInterval(redisProperties.getPoolCleanerInterval());
                // password
                if (StrUtil.isNotBlank(redisProperties.getPassword())) {
                    options.setPassword(redisProperties.getPassword());
                }
                // connect address [redis://localhost:6379/0, redis://localhost:6779/1]
                for (String url : redisProperties.getUrls()) {
                    options.addConnectionString(url);
                }
                // sentinel
                if (redisProperties.getClientType().equals(RedisClientType.SENTINEL)) {
                    options.setRole(redisProperties.getRole()).setMasterName(redisProperties.getMasterName());
                }
                //创建redis实例
                redis = Redis.createClient(vertx, options);
                redisAPI = RedisAPI.api(redis);
            }
            handler.handle(Future.succeededFuture());
        } catch (Exception e) {
            handler.handle(Future.failedFuture(e));
        }
    }

}
